/*******************************************************************/
/*                                                                 */
/* This program calls the various REXX external functions          */
/* provided in the gbmrx.c function package.                       */
/*                                                                 */
/*******************************************************************/
/* '@echo off' */

call RxFuncAdd 'GBM_LoadFuncs', 'GBMRX', 'GBM_LoadFuncs'
call GBM_LoadFuncs

/* Set the filename used in this example */
readFilename  = "images\example.gif"
readOptions   = "index=5,ext_bpp"  /* allow deep colour depth > 24bpp */

writeFilename = "images\example_out.png"
writeOptions  = "compression=9"

/* Get version number of GBMRX lib ***/
gbmVersionRexx = GBM_VersionRexx()
say 'GBMRX.DLL version: 'gbmVersionRexx

/* Get version number of GBM.DLL ***/
gbmVersion = GBM_Version()
say 'GBM.DLL   version: 'gbmVersion

/* Get the supported types with all descriptions */
rc = GBM_Types('types.')
if rc <> "0" then
do
   say
   say '### Cannot query supported bitmap types.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say
   say '======================'
   say 'Number of supported formats: 'types.0
   say '----------------------'
   do i = 1 to types.0
      say '  Extensions: 'types.i.extensions
      say '  Short desc: 'types.i.shortname
      say '  Long  desc: 'types.i.longname
      if i < types.0 then
      do
        say '----------------------'
      end
   end
end

say
say '======================'
say '======================'
say 'Checking file: "'readFilename'"'

/* Get file type */
rc = GBM_FileType(readFilename, 'type.')
if rc <> "0" then
do
   say
   say '### Cannot detect bitmap type.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say '----------------------'
   say '  Extensions: 'type.extensions
   say '  Short desc: 'type.shortname
   say '  Long  desc: 'type.longname
end

/* Get number of pages */
numPages = GBM_FilePages(readFilename)
say '----------------------'
say 'Number of pages: 'numPages

/* Get bitmap header */
rc = GBM_FileHeader(readFilename, readOptions, 'header.')
if rc <> "0" then
do
   say
   say '### Cannot read bitmap header.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say '----------------------'
   say '  Bitmap width : 'header.width
   say '  Bitmap height: 'header.height
   say '  Bitmap bpp   : 'header.bpp' bpp'
end

/* Get bitmap palette */
rc = GBM_FilePalette(readFilename, readOptions, 'colors.')
if rc <> "0" then
do
   say
   say '### Cannot read bitmap palette.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   /* Print palette entries. Note the palette can be empty for true color bitmaps. */
   say
   say '======================'
   say 'Number of palette entries: 'colors.0
   say '----------------------'
   do i = 1 to colors.0
      say '  Index, (R,G,B): 'i', ('colors.i.red','colors.i.green','colors.i.blue')'
   end
end

/* Get bitmap data */
say
say '======================'
say 'Reading bitmap data'

rc = GBM_FileData(readFilename, readOptions, 'data')
if rc <> "0" then
do
   say
   say '### Cannot read bitmap data.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say '----------------------'
   expectedLength = (TRUNC((header.width * header.bpp + 31)/32)*4)*header.height
   say '  Bitmap data length (expected): 'expectedLength
   say '  Bitmap data length (read)    : 'LENGTH(data)
end

/* ----------------------------------------------- */
say
say '======================'
say 'Writing bitmap to file: "'writeFilename'"'

/* Get output file type */
rc = GBM_FileType(writeFilename, 'typeout.')
if rc <> "0" then
do
   say
   say '### Cannot detect output bitmap type.'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say '----------------------'
   say '  Extensions: 'typeout.extensions
   say '  Short desc: 'typeout.shortname
   say '  Long  desc: 'typeout.longname
end

/* check if output colour depth is support by the write format */

rc = GBM_IsBppSupported(WORD(typeout.extensions, 1), header.bpp, 'w')
if rc = "0" then
do
   say
   say '### Colour depth is not supported by output format'
   call GBM_DropFuncs
   exit(1)
end

/* Write new bitmap in other format */

rc = GBM_FileWrite(writeFilename, writeOptions, 'header.', 'colors.', 'data')
if rc <> "0" then
do
   say
   say '### Cannot write bitmap "'writeFilename'".'
   say rc
   call GBM_DropFuncs
   exit(1)
end
else
do
   say '----------------------'
   say '  SUCCESS'
end

say
say 'All is DONE.'

call GBM_DropFuncs
exit(0)


